package moteur;

public class Moteur {

	/**
	 * le jeu a lancer
	 */
	private Jeu jeu;

	/**
	 * constructeur de moteur
	 * 
	 * @param j
	 *            le jeu a executer
	 */
	public Moteur(Jeu j) {
		this.jeu = j;
	}

	/**
	 * lance le jeu pendant un certain temps
	 * 
	 * @param nbIter
	 *            le nombre d'iterations
	 * @param dureeBoucleMs
	 *            la duree d'une iteration en mSecondes
	 */
	public void lancerBoucle(int nbIter, long dureeBoucleMs) {
		for (int i = 0; i < nbIter; i++) {
			long dateDebut = System.nanoTime();
			
			double dt = (1.0 * dureeBoucleMs) / 1000;
			jeu.update(dt);
			jeu.render();

			// apres le render en nanos
			long dateFin = System.nanoTime();

			// duree en nanos
			long dureeEcoulee = dateFin - dateDebut;
			long dureeNs = dureeBoucleMs * 1000000L;
			long dureeRestante = dureeNs - dureeEcoulee;
			// System.out.println("doit attendre" + dureeRestante / 1000L);

			this.attendre(dureeRestante);
		}
	}

	/**
	 * on attend la duree restante
	 * 
	 * @param dureeRestante
	 *            duree a attendre
	 * 
	 * @throws AssertionError
	 *             en cas de soucis de temps
	 */
	private void attendre(long dureeRestante) throws AssertionError {
		// attend dans le vide
		long debutAttente = System.nanoTime();
		if (dureeRestante < 0)
			throw new AssertionError("pas assez de vitesse");
		// on attend le temps qu'il faut
		while (System.nanoTime() - debutAttente < dureeRestante) {
		}
	}

}
